"""Module for IQ option websocket."""

def digital_option_placed(api, message, api_dict_clean):
    if message["name"] == "digital-option-placed":
        if message["msg"].get("id") != None:
            api_dict_clean(api.digital_option_placed_id)
            api.digital_option_placed_id[message["request_id"]
                                                ] = message["msg"]["id"]
        else:
            api.digital_option_placed_id[message["request_id"]] = {
                "code": "error_place_digital_order",
                "message": message["msg"]["message"]
            }


"""Module for IQ option websocket."""

import json
import logging
import websocket
import iqoptionapi.constants as OP_code
import iqoptionapi.global_value as global_value
from threading import Thread
from iqoptionapi.ws.received.technical_indicators import technical_indicators
from iqoptionapi.ws.received.time_sync import time_sync
from iqoptionapi.ws.received.heartbeat import heartbeat
from iqoptionapi.ws.received.balances import balances
from iqoptionapi.ws.received.profile import profile
from iqoptionapi.ws.received.balance_changed import balance_changed
from iqoptionapi.ws.received.candles import candles
from iqoptionapi.ws.received.buy_complete import buy_complete
from iqoptionapi.ws.received.option import option
from iqoptionapi.ws.received.position_history import position_history
from iqoptionapi.ws.received.list_info_data import list_info_data
from iqoptionapi.ws.received.candle_generated import candle_generated_realtime
from iqoptionapi.ws.received.candle_generated_v2 import candle_generated_v2
from iqoptionapi.ws.received.commission_changed import commission_changed
from iqoptionapi.ws.received.socket_option_opened import socket_option_opened
from iqoptionapi.ws.received.api_option_init_all_result import api_option_init_all_result
from iqoptionapi.ws.received.initialization_data import initialization_data
from iqoptionapi.ws.received.underlying_list import underlying_list
from iqoptionapi.ws.received.instruments import instruments
from iqoptionapi.ws.received.financial_information import financial_information
from iqoptionapi.ws.received.position_changed import position_changed
from iqoptionapi.ws.received.option_opened import option_opened
from iqoptionapi.ws.received.option_closed import option_closed
from iqoptionapi.ws.received.top_assets_updated import top_assets_updated
from iqoptionapi.ws.received.strike_list import strike_list
from iqoptionapi.ws.received.api_game_betinfo_result import api_game_betinfo_result
from iqoptionapi.ws.received.traders_mood_changed import traders_mood_changed
from iqoptionapi.ws.received.order import order
from iqoptionapi.ws.received.position import position
from iqoptionapi.ws.received.positions import positions
from iqoptionapi.ws.received.order_placed_temp import order_placed_temp
from iqoptionapi.ws.received.deferred_orders import deferred_orders
from iqoptionapi.ws.received.history_positions import history_positions
from iqoptionapi.ws.received.available_leverages import available_leverages
from iqoptionapi.ws.received.order_canceled import order_canceled
from iqoptionapi.ws.received.position_closed import position_closed
from iqoptionapi.ws.received.overnight_fee import overnight_fee
from iqoptionapi.ws.received.api_game_getoptions_result import api_game_getoptions_result
from iqoptionapi.ws.received.sold_options import sold_options
from iqoptionapi.ws.received.tpsl_changed import tpsl_changed
from iqoptionapi.ws.received.auto_margin_call_changed import auto_margin_call_changed
from iqoptionapi.ws.received.digital_option_placed import digital_option_placed
from iqoptionapi.ws.received.result import result
from iqoptionapi.ws.received.instrument_quotes_generated import instrument_quotes_generated
from iqoptionapi.ws.received.training_balance_reset import training_balance_reset
from iqoptionapi.ws.received.socket_option_closed import socket_option_closed
from iqoptionapi.ws.received.live_deal_binary_option_placed import live_deal_binary_option_placed
from iqoptionapi.ws.received.live_deal_digital_option import live_deal_digital_option
from iqoptionapi.ws.received.leaderboard_deals_client import leaderboard_deals_client
from iqoptionapi.ws.received.live_deal import live_deal
from iqoptionapi.ws.received.user_profile_client import user_profile_client
from iqoptionapi.ws.received.leaderboard_userinfo_deals_client import leaderboard_userinfo_deals_client
from iqoptionapi.ws.received.client_price_generated import client_price_generated
from iqoptionapi.ws.received.users_availability import users_availability


class WebsocketClient(object):
    """Class for work with IQ option websocket."""

    def __init__(self, api):
        """
        :param api: The instance of :class:`IQOptionAPI
            <iqoptionapi.api.IQOptionAPI>`.
        """
        self.api = api
        self.wss = websocket.WebSocketApp(
            self.api.wss_url, on_message=self.on_message,
            on_error=self.on_error, on_close=self.on_close,
            on_open=self.on_open)

    def dict_queue_add(self, dict, maxdict, key1, key2, key3, value):
        if key3 in dict[key1][key2]:
            dict[key1][key2][key3] = value
        else:
            while True:
                try:
                    dic_size = len(dict[key1][key2])
                except:
                    dic_size = 0
                if dic_size < maxdict:
                    dict[key1][key2][key3] = value
                    break
                else:
                    # del mini key
                    del dict[key1][key2][sorted(
                        dict[key1][key2].keys(), reverse=False)[0]]

    def api_dict_clean(self, obj):
        if len(obj) > 5000:
            for k in obj.keys():
                del obj[k]
                break

    def on_message(self, message):  # pylint: disable=unused-argument
        """Method to process websocket messages."""
        global_value.ssl_Mutual_exclusion = True
        logger = logging.getLogger(__name__)
        logger.debug(message)

        message = json.loads(str(message))


        technical_indicators(self.api, message, self.api_dict_clean)
        time_sync(self.api, message)
        heartbeat(self.api, message)
        balances(self.api, message)
        profile(self.api, message)
        balance_changed(self.api, message)
        candles(self.api, message)
        buy_complete(self.api, message)
        option(self.api, message)
        position_history(self.api, message)
        list_info_data(self.api, message)
        candle_generated_realtime(self.api, message, self.dict_queue_add)
        candle_generated_v2(self.api, message, self.dict_queue_add)
        commission_changed(self.api, message)
        socket_option_opened(self.api, message)
        api_option_init_all_result(self.api, message)
        initialization_data(self.api, message)
        underlying_list(self.api, message)
        instruments(self.api, message)
        financial_information(self.api, message)
        position_changed(self.api, message)
        option_opened(self.api, message)
        option_closed(self.api, message)
        top_assets_updated(self.api, message)
        strike_list(self.api, message)
        api_game_betinfo_result(self.api, message)
        traders_mood_changed(self.api, message)
         # ------for forex&cfd&crypto..
        order_placed_temp(self.api, message)
        order(self.api, message)
        position(self.api, message)
        positions(self.api, message)
        order_placed_temp(self.api, message)
        deferred_orders(self.api, message)
        history_positions(self.api, message)
        available_leverages(self.api, message)
        order_canceled(self.api, message)
        position_closed(self.api, message)
        overnight_fee(self.api, message)
        api_game_getoptions_result(self.api, message)
        sold_options(self.api, message)
        tpsl_changed(self.api, message)
        auto_margin_call_changed(self.api, message)
        digital_option_placed(self.api, message, self.api_dict_clean)
        result(self.api, message)
        instrument_quotes_generated(self.api, message)
        training_balance_reset(self.api, message)
        socket_option_closed(self.api, message)
        live_deal_binary_option_placed(self.api, message)
        live_deal_digital_option(self.api, message)
        leaderboard_deals_client(self.api, message)
        live_deal(self.api, message)
        user_profile_client(self.api, message)
        leaderboard_userinfo_deals_client(self.api, message)
        users_availability(self.api, message)
        client_price_generated(self.api, message)

        global_value.ssl_Mutual_exclusion = False

    @staticmethod
    def on_error(wss, error):  # pylint: disable=unused-argument
        """Method to process websocket errors."""
        logger = logging.getLogger(__name__)
        logger.error(error)
        global_value.websocket_error_reason = str(error)
        global_value.check_websocket_if_error = True

    @staticmethod
    def on_open(wss):  # pylint: disable=unused-argument
        """Method to process websocket open."""
        logger = logging.getLogger(__name__)
        logger.debug("Websocket client connected.")
        global_value.check_websocket_if_connect = 1

    @staticmethod
    def on_close(wss):  # pylint: disable=unused-argument
        """Method to process websocket close."""
        logger = logging.getLogger(__name__)
        logger.debug("Websocket connection closed.")
        global_value.check_websocket_if_connect = 0



"""Module for IQ option websocket."""

def candles(api, message):
    if message['name'] == 'candles':
        try:
            api.candles.candles_data = message["msg"]["candles"]
        except:
            pass


"""Module for IQ Option Profile websocket object."""
from iqoptionapi.ws.objects.base import Base


class Profile(Base):
    """Class for IQ Option Profile websocket object."""

    def __init__(self):
        super(Profile, self).__init__()
        self.__name = "profile"
        self.__skey = None
        self.__balance = None
        self.__balance_id=None
        self.__balances=None
        self.__msg=None
        self.__currency=None
        self.__minimum_amount=1
        self.__balance_type=None
        self.__currency_char=None
        self.__time_zone=-3

    @property
    def skey(self):
        """Property to get skey value.

        :returns: The skey value.
        """
        return self.__skey

    @skey.setter
    def skey(self, skey):
        """Method to set skey value."""
        self.__skey = skey
#----------------------------------------------------------------
    @property
    def balance(self):
        """Property to get balance value.

        :returns: The balance value.
        """
        return self.__balance

    @balance.setter
    def balance(self, balance):
        """Method to set balance value."""
        self.__balance = balance

#--------------------------------------------------------------------- 
    @property
    def balance_id(self):
        """Property to get balance value.

        :returns: The balance value.
        """
        return self.__balance_id
    @balance_id.setter
    def balance_id(self, balance_id):
        """Method to set balance value."""
        self.__balance_id = balance_id


#------------------------------------------------------------------------
    @property
    def balance_type(self):
        """Property to get balance value.

        :returns: The balance value.
        """
        return self.__balance_type
    @balance_type.setter
    def balance_type(self, balance_type):
        """Method to set balance value."""
        self.__balance_type = balance_type




#----------------------------------------
    @property
    def balances(self):
        """Property to get balance value.

        :returns: The balance value.
        """
        return self.__balances
    @balances.setter
    def balances(self, balances):
        """Method to set balance value."""
        self.__balances = balances

#------------
    @property
    def msg(self):
        return self.__msg
    @msg.setter
    def msg(self, msg):
        self.__msg = msg

#------------
    @property
    def currency(self):
        return self.__currency
    
    @currency.setter
    def currency(self, currency):
        self.__currency = currency
        if self.__currency.upper() == "BRL":
            self.__minimum_amount = 2

    @property
    def minimum_amount(self):
        return self.__minimum_amount
#------------
    @property
    def currency_char(self):
        return self.__currency_char

    @currency_char.setter
    def currency_char(self, currency_char):
        self.__currency_char = currency_char
#------------
    @property
    def time_zone(self):
        return self.__time_zone

    @time_zone.setter
    def time_zone(self, time_zone):
        self.__time_zone = int(time_zone/60)


"""Module for IQ option websocket."""
import iqoptionapi.global_value as global_value

def profile(api, message):
    if message["name"] == "profile":
        api.profile.msg = message["msg"]
        if api.profile.msg != False:
            # ---------------------------
            try:
                api.profile.balance = message["msg"]["balance"]
            except:
                pass
            # Set Default account
            if global_value.balance_id == None:
                for balance in message["msg"]["balances"]:
                    if balance["type"] == 4:
                        global_value.balance_id = balance["id"]
                        break
            try:
                api.profile.balance_id = message["msg"]["balance_id"]
            except:
                pass

            try:
                api.profile.balance_type = message["msg"]["balance_type"]
            except:
                pass

            try:
                api.profile.balances = message["msg"]["balances"]
            except:
                pass


"""Module for IQ option websocket."""

def list_info_data(api, message):
    if message["name"] == "listInfoData":
        for get_m in message["msg"]:
            api.listinfodata.set(get_m["win"], get_m["game_state"], get_m["id"])


"""Module for IQ option websocket."""
import iqoptionapi.constants as OP_code
import iqoptionapi.global_value as global_value

def candle_generated_realtime(api, message, dict_queue_add):
    if message["name"] == "candle-generated":
        Active_name = list(OP_code.ACTIVES.keys())[list(
            OP_code.ACTIVES.values()).index(message["msg"]["active_id"])]

        active = str(Active_name)
        size = int(message["msg"]["size"])
        from_ = int(message["msg"]["from"])
        msg = message["msg"]
        maxdict = api.real_time_candles_maxdict_table[Active_name][size]

        dict_queue_add(api.real_time_candles,
                            maxdict, active, size, from_, msg)
        api.candle_generated_check[active][size] = True


import datetime
import time
from iqoptionapi.ws.chanels.base import Base


class Traders_mood_subscribe(Base):
    name = "subscribeMessage"

    def __call__(self, active, instrument="turbo-option"):
        data = {
            "name": "traders-mood-changed",
            "params":
                {
                    "routingFilters":
                        {
                            "instrument": instrument,
                            "asset_id": active
                        }
                }

        }

        self.send_websocket_request(self.name, data)


class Traders_mood_unsubscribe(Base):
    name = "unsubscribeMessage"

    def __call__(self, active, instrument="turbo-option"):
        data = {
            "name": "traders-mood-changed",
            "params":
                {
                    "routingFilters":
                        {
                            "instrument": instrument,
                            "asset_id": active
                        }
                }

        }

        self.send_websocket_request(self.name, data)



"""Module for IQ option websocket."""

def position_changed(api, message):
    if message["name"] == "position-changed":
        if message["microserviceName"] == "portfolio" and (message["msg"]["source"] == "digital-options") or message["msg"]["source"] == "trading":
            api.order_async[int(message["msg"]["raw_event"]["order_ids"][0])][message["name"]] = message
        elif message["microserviceName"] == "portfolio" and message["msg"]["source"] == "binary-options":
            api.order_async[int(message["msg"]["external_id"])][message["name"]] = message
        else:
            api.position_changed = message


import datetime
import time
from iqoptionapi.ws.chanels.base import Base


class Technical_indicators(Base):
    name = "sendMessage"

    def __call__(self, active):
        data = {
            "name": "trading-signals.get-technical-indicators",
            "version": "1.0",
            "body": {
                "id": active
            }
        }
        request_id = str(time.time()).split('.')[1]
        self.send_websocket_request(self.name, data, request_id)
        return request_id



 
from iqoptionapi.ws.chanels.base import Base
import time

class Sell_Digital_Option(Base):
    name = "sendMessage"
    def __call__(self, position_ids):
        """ 
        :param options_ids: list or int
        """
        if type(position_ids) == list:
                id_list=[]
                id_list.append(position_ids)
                position_ids=id_list
        
                data = {"name":"digital-options.close-position-batch",
                        "version":"1.0",
                        "body":{
                                "position_ids":(position_ids)
                                }
                        }
        else:
                data = {"name":"digital-options.close-position",
                        "version":"1.0",
                        "body":{
                                "position_id":position_ids
                                }
                        }
        request_id = int(str(time.time()).split('.')[1])
        self.send_websocket_request(self.name, data, request_id)



"""Module for IQ option websocket."""

def api_game_betinfo_result(api, message):
    if message["name"] == "api_game_betinfo_result":
        try:
            api.game_betinfo.isSuccessful = message["msg"]["isSuccessful"]
            api.game_betinfo.dict = message["msg"]
        except:
            pass


"""Module for IQ option websocket."""

def technical_indicators(api, message, api_dict_clean):
    if message["name"] == "technical-indicators":
        if message["msg"].get("indicators") != None:
            api_dict_clean(api.technical_indicators)
            api.technical_indicators[message["request_id"]] = message["msg"]["indicators"]
        else:
            api.technical_indicators[message["request_id"]] = {
                "code": "no_technical_indicator_available",
                "message": message["msg"]["message"]
            }


"""Module for IQ option websocket."""
import iqoptionapi.constants as OP_code
from threading import Thread

def live_deal(api, message): 
    if message["name"] == "live-deal":
        # name = message["name"]
        active_id = message["msg"]["instrument_active_id"]
        active = list(OP_code.ACTIVES.keys())[
            list(OP_code.ACTIVES.values()).index(active_id)]
        _type = message["msg"]["instrument_type"]
        try:
            # api.live_deal_data[name][active][_type].appendleft(
            #     message["msg"])
            if hasattr(api.live_deal_cb, '__call__'):
                cb_data = {
                    "active": active,
                    **message["msg"]
                }
                livedeal = Thread(target=api.live_deal_cb,
                                kwargs=(cb_data))
                livedeal.daemon = True
                livedeal.start()
        except:
            pass


import iqoptionapi.constants as OP_code

def candle_generated_v2(api, message, dict_queue_add):
    if message["name"] == "candles-generated":
        Active_name = list(OP_code.ACTIVES.keys())[list(
                OP_code.ACTIVES.values()).index(message["msg"]["active_id"])]
        active = str(Active_name)
        for k, v in message["msg"]["candles"].items():
            v["active_id"] = message["msg"]["active_id"]
            v["at"] = message["msg"]["at"]
            v["ask"] = message["msg"]["ask"]
            v["bid"] = message["msg"]["bid"]
            v["close"] = message["msg"]["value"]
            v["size"] = int(k)
            size = int(v["size"])
            from_ = int(v["from"])
            maxdict = api.real_time_candles_maxdict_table[Active_name][size]
            msg = v
            dict_queue_add(api.real_time_candles, maxdict, active, size, from_, msg)

        api.candle_generated_all_size_check[active] = True


"""Module for IQ option websocket."""

def top_assets_updated(api, message):
    if message["name"] == "top-assets-updated":
        api.top_assets_updated_data[str(message["msg"]["instrument_type"])] = message["msg"]["data"]


"""Module for IQ option websocket."""

def buy_complete(api, message):
    if message['name'] == 'buyComplete':
        try:
            api.buy_successful = message["msg"]["isSuccessful"]
            api.buy_id = message["msg"]["result"]["id"]
        except:
            pass


"""Module for IQ option websocket."""

def client_price_generated(api, message):
    if message["name"] == "client-price-generated":
        ask_price = [d for d in message["msg"]["prices"] if d['strike'] == 'SPT'][0]['call']['ask']
        api.digital_payout = int(((100-ask_price)*100)/ask_price)
        api.client_price_generated = message["msg"]
    else:
        pass


"""Module for IQ option websocket."""

def training_balance_reset(api, message):
    if message["name"] == "training-balance-reset":
        api.training_balance_reset_request = message["msg"]["isSuccessful"]


"""Module for IQ Option buyV2 websocket chanel."""
from datetime import datetime, timedelta
import iqoptionapi.global_value as global_value
from iqoptionapi.ws.chanels.base import Base
from iqoptionapi.expiration import get_expiration_time


class Buyv2(Base):
    """Class for IQ option buy websocket chanel."""
    # pylint: disable=too-few-public-methods

    name = "sendMessage"

    def __call__(self, price, active, direction, duration):
        """Method to send message to buyv2 websocket chanel.
        :param price: The buying price.
        :param active: The buying active.
        :param direction: The buying direction.
        """

        exp, idx = get_expiration_time(
            int(self.api.timesync.server_timestamp), duration)

        if idx < 5:
            option = 3  # turbo
        else:
            option = 1  # non-turbo / binary

        data = {
            "price": price,
            "act": active,
            "exp": int(exp),
            "type": option,
            "direction": direction.lower(),
            "user_balance_id": int(global_value.balance_id),
            "time": self.api.timesync.server_timestamp
        }

        self.send_websocket_request(self.name, data)



"""Module for IQ option websocket."""

def traders_mood_changed(api, message):
    if message["name"] == "traders-mood-changed":
        api.traders_mood[message["msg"]["asset_id"]] = message["msg"]["value"]


"""Module for IQ option websocket."""

def leaderboard_userinfo_deals_client(api, message):
    if message["name"] == "leaderboard-userinfo-deals-client":
        api.leaderboard_userinfo_deals_client = message["msg"]


"""Module for IQ option websocket."""

def socket_option_opened(api, message):
    if message["name"] == "socket-option-opened":
        id = message["msg"]["id"]
        api.socket_option_opened[id] = message


"""Module for IQ option websocket."""
import iqoptionapi.constants as OP_code

def instrument_quotes_generated(api, message):
    if message["name"] == "instrument-quotes-generated":

        Active_name = list(OP_code.ACTIVES.keys())[list(OP_code.ACTIVES.values()).index(message["msg"]["active"])]
        period = message["msg"]["expiration"]["period"]
        ans = {}
        for data in message["msg"]["quotes"]:
            # FROM IQ OPTION SOURCE CODE
            if data["price"]["ask"] == None:
                ProfitPercent = None
            else:
                askPrice = (float)(data["price"]["ask"])
                ProfitPercent = ((100-askPrice)*100)/askPrice

            for symble in data["symbols"]:
                try:
                    """
                    ID SAMPLE:doUSDJPY-OTC201811111204PT1MC11350481
                    """

                    """
                    dict ID-prodit:{ID:profit}
                    """

                    ans[symble] = ProfitPercent
                except:
                    pass
        api.instrument_quites_generated_timestamp[Active_name][
            period] = message["msg"]["expiration"]["timestamp"]
        api.instrument_quites_generated_data[Active_name][period] = ans

        api.instrument_quotes_generated_raw_data[Active_name][period] = message


"""Module for IQ option websocket."""

def option_closed(api, message):
    if message["name"] == "option-closed":
        api.order_async[int(message["msg"]["option_id"])][message["name"]] = message
        if message["microserviceName"] == "binary-options":
            api.order_binary[message["msg"]["option_id"]] = message['msg']


"""The python wrapper for IQ Option API package setup."""
from setuptools import setup, find_packages
#from myiqoptionapi.iqoptionapi.version_control import api_version

setup(
    name="my-iqoptionapi",
    version="7.1.1",
    packages=find_packages(),
    install_requires=["pylint", "requests", "websocket-client==0.56"],
    include_package_data=True,
    description="Best IQ Option API for python",
    long_description="Best IQ Option API for python",
    url="https://github.com/zskull-py/iqoptionapi",
    author="zSkull.py",
    keywords="iqoption",
    zip_safe=False
)



"""Module for IQ option websocket."""

def balance_changed(api, message):
    if message['name'] == 'balance-changed':
        balance = message['msg']['current_balance']
        # if self.api.get_active_account_type() == balance['type']:
        try:
            api.profile.balance = balance["amount"]
        except:
            pass

        try:
            api.profile.balance_id = balance["id"]
        except:
            pass

        try:
            api.profile.balance_type = balance["type"]
        except:
            pass


"""Module for IQ option websocket."""

def position_closed(api, message):
    if message["name"] == "position-closed":
        api.close_position_data = message
        api.sold_digital_options_respond = message


"""Module for IQ option websocket."""
import iqoptionapi.constants as OP_code

def commission_changed(api, message):
    if message["name"] == "commission-changed":
        instrument_type = message["msg"]["instrument_type"]
        active_id = message["msg"]["active_id"]
        Active_name = list(OP_code.ACTIVES.keys())[list(
            OP_code.ACTIVES.values()).index(active_id)]
        commission = message["msg"]["commission"]["value"]
        api.subscribe_commission_changed_data[instrument_type][Active_name][api.timesync.server_timestamp] = int(
            commission)


"""Module for IQ Option http verify resource."""

from iqoptionapi.http.resource import Resource
import json


class Verify(Resource):
    """Class for IQ option verify resource."""
    # pylint: disable=too-few-public-methods

    url = ""

    def _post(self, data=None, headers=None):
        """Send get request for IQ Option API verify http resource.

        :returns: The instance of :class:`requests.Response`.
        """
        return self.api.send_http_request_v2(method="POST", url="https://auth.iqoption.com/api/v2/verify/2fa",data=json.dumps(data), headers=headers)

    def __call__(self, sms_received, token_sms):
        """Method to get IQ Option API verify http request.

        :param str sms_received: The sms received of a IQ Option server 2FA.
        :param str token_sms: The token of a IQ Option server 2FA.

        :returns: The instance of :class:`requests.Response`.
        """
        data = {"code": str(sms_received),
                "token": token_sms}

        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Referer': 'https://iqoption.com/en/login',
            'Sec-Fetch-Mode': 'cors',
            'User-Agent': 'Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36'
            }

        return self._post(data=data, headers=headers)



# python

import datetime
import time
from iqoptionapi.ws.chanels.base import Base
import iqoptionapi.global_value as global_value
from random import randint
# work for forex digit cfd(stock)


class Digital_options_place_digital_option(Base):
    name = "sendMessage"

    def __call__(self, instrument_id, amount, user_balance_id=None):
        if user_balance_id == None:
            user_balance_id = int(self.api.profile.balance_id)

        data = {
            "name": "digital-options.place-digital-option",
            "version": "1.0",
            "body": {
                "user_balance_id": int(global_value.balance_id),
                "instrument_id": str(instrument_id),
                "amount": str(amount)
            }
        }
        request_id = str(randint(0, 100000))
        self.send_websocket_request(self.name, data, request_id)
        return request_id


class Digital_options_close_position(Base):
    name = "sendMessage"

    def __call__(self, position_id):
        data = {
            "name": "digital-options.close-position",
            "version": "1.0",
            "body": {
                "position_id": int(position_id)
            }
        }
        self.send_websocket_request(self.name, data)


class DigitalOptionsPlaceDigitalOptionV2(Base):
    name = "sendMessage"

    def __call__(self, instrument_id, asset_id, amount):
        data = {
            "name": "digital-options.place-digital-option",
            "version": "2.0",
            "body": {
                "amount": str(amount),
                "asset_id": int(asset_id),
                "instrument_id": instrument_id,
                "instrument_index": 0,
                "user_balance_id": int(global_value.balance_id)
            }
        }

        request_id = str(randint(0, 100000))
        self.send_websocket_request(self.name, data, request_id)

        return request_id



"""Module for IQ option unsubscribe websocket chanel."""

from iqoptionapi.ws.chanels.base import Base
import datetime
import iqoptionapi.constants as OP_code


class Unsubscribe(Base):
    """Class for IQ option candles websocket chanel."""
    # pylint: disable=too-few-public-methods

    name = "unsubscribeMessage"

    def __call__(self, active_id, size=1):

        data = {"name": "candle-generated",
                "params": {
                    "routingFilters": {
                        "active_id": str(active_id),
                        "size": int(size)
                    }
                }
                }

        self.send_websocket_request(self.name, data)


class Unsubscribe_candles(Base):
    """Class for IQ option candles websocket chanel."""
    # pylint: disable=too-few-public-methods

    name = "unsubscribeMessage"

    def __call__(self, active_id, size=1):

        data = {"name": "candles-generated",
                "params": {
                    "routingFilters": {
                        "active_id": str(active_id)
                    }
                }
                }

        self.send_websocket_request(self.name, data)


class Unsubscribe_Instrument_Quites_Generated(Base):
    name = "unsubscribeMessage"

    def __call__(self, ACTIVE, expiration_period):
        data = {
            "name": "instrument-quotes-generated",
            "params": {
                "routingFilters": {
                    "active": int(OP_code.ACTIVES[ACTIVE]),
                    "expiration_period": int(expiration_period*60),
                    "kind": "digital-option",
                },
            },
            "version": "1.0"
        }
        self.send_websocket_request(self.name, data)

    def get_digital_expiration_time(self, duration):
        exp = int(self.api.timesync.server_timestamp)
        value = datetime.datetime.fromtimestamp(exp)
        minute = int(value.strftime('%M'))
        # second=int(value.strftime('%S'))
        ans = exp-exp % 60  # delete second
        ans = ans+(duration-minute % duration)*60
        if exp > ans-10:
            ans = ans+(duration)*60

        return ans


class Unsubscribe_top_assets_updated(Base):
    name = "unsubscribeMessage"

    def __call__(self, instrument_type):

        data = {"name": "top-assets-updated",
                "params": {
                    "routingFilters": {
                        "instrument_type": str(instrument_type)

                    }
                },
                "version": "1.2"
                }
        self.send_websocket_request(self.name, data)


class Unsubscribe_commission_changed(Base):
    name = "unsubscribeMessage"
    def __call__(self, instrument_type):

        data = {"name": "commission-changed",
                "params": {
                    "routingFilters": {
                        "instrument_type": str(instrument_type)
                    }
                },
                "version": "1.0"
                }
        self.send_websocket_request(self.name, data)


class Unscribe_live_deal(Base):
    name = "unsubscribeMessage"

    def __call__(self, name, active_id, _type):
        if name == "live-deal-binary-option-placed":
            _type_name = "option_type"
            _active_id = "active_id"
        elif name == "live-deal-digital-option":
            _type_name = "expiration_type"
            _active_id = "instrument_active_id"
        elif name == "live-deal":
            _type_name = "instrument_type"
            _active_id = "instrument_active_id"

        data = {"name": str(name),
                "params": {
            "routingFilters": {
                _active_id: int(active_id),
                _type_name: str(_type)
            }
        },
            "version": "2.0"
        }
        self.send_websocket_request(self.name, data)


class UnsubscribeDigitalPriceSplitter(Base):
    name = "unsubscribeMessage"

    def __call__(self, asset_id):
        data = {
            "name": "price-splitter.client-price-generated",
            "version": "1.0",
            "params": {
                "routingFilters": {
                    "instrument_type": "digital-option",
                    "asset_id": int(asset_id)
                }
            }
        }

        self.send_websocket_request(self.name, msg=data)
